
Uses Crt, Graph13h;
{ Sample program to display a 320x200x256 PCX in
  mode 13h.  PCX source copied from MCGA07, a MCGA
  graphics unit written by James Cook in his MCGA
  programming tutorial on Quantum Leap BBS }

TYPE
  TPalette = array[0..767] of Byte;
  PalettePtr = ^TPalette;
{ PCX stuff }
  PCXHeaderPtr=  ^PCXHeader;
  PCXHeader   =  record
                   Signature      :  Char;
                   Version        :  Char;
                   Encoding       :  Char;
                   BitsPerPixel   :  Char;
                   XMin,YMin,
                   XMax,YMax      :  Integer;
                   HRes,VRes      :  Integer;
                   Palette        :  Array [0..47] of byte;
                   Reserved       :  Char;
                   Planes         :  Char;
                   BytesPerLine   :  Integer;
                   PaletteType    :  Integer;
                   Filler         :  Array [0..57] of byte;
                 end;

Procedure ExtractLineASM (BytesWide:Integer;Var Source,Dest:Pointer);
var
  DestSeg,
  DestOfs,
  SourceSeg,
  SourceOfs   :  Word;
begin
  SourceSeg := Seg (Source^);
  SourceOfs := Ofs (Source^);
  DestSeg   := Seg (Dest^);
  DestOfs   := Ofs (Dest^);

  asm
    push  ds
    push  si

    cld

    mov   ax,DestSeg
    mov   es,ax
    mov   di,DestOfs     { es:di -> destination pointer }
    mov   ax,SourceSeg
    mov   ds,ax
    mov   si,SourceOfs   { ds:si -> source buffer }

    mov   bx,di
    add   bx,BytesWide   { bx holds position to stop for this row }
    xor   cx,cx

  @@GetNextByte:
    cmp   bx,di          { are we done with the line }
    jbe   @@ExitHere

    lodsb                { al contains next byte }

    mov   ah,al
    and   ah,0C0h
    cmp   ah,0C0h

    jne    @@SingleByte
                         { must be a run of bytes }
    mov   cl,al
    and   cl,3Fh
    lodsb
    rep   stosb
    jmp   @@GetNextByte

  @@SingleByte:
    stosb
    jmp   @@GetNextByte

  @@ExitHere:
    mov   SourceSeg,ds
    mov   SourceOfs,si
    mov   DestSeg,es
    mov   DestOfs,di

    pop   si
    pop   ds
  end;

  Source := Ptr (SourceSeg,SourceOfs);
  Dest   := Ptr (DestSeg,DestOfs);
end;

Procedure DisplayPCX (X,Y:Integer;Buf:Pointer);
var
  I,NumRows,
  BytesWide   :  Integer;
  Header      :  PCXHeaderPtr;
  DestPtr     :  Pointer;
  Offset      :  Word;

begin
  Header    := Ptr (Seg(Buf^),Ofs(Buf^));
  Buf       := Ptr (Seg(Buf^),Ofs(Buf^)+128);
  Offset    := Y * 320 + X;
  NumRows   := Header^.YMax - Header^.YMin + 1;
  BytesWide := Header^.XMax - Header^.XMin + 1;
  If Odd (BytesWide) then Inc (BytesWide);

  For I := 1 to NumRows do begin
    DestPtr := Ptr ($A000,Offset);
    ExtractLineASM (BytesWide,Buf,DestPtr);
    Inc (Offset,320);
    end;
end;
{ end PCX stuff }





  procedure DrawFrame(x,y,dx,dy:integer);
    var i,c:integer;
    begin
      for i:=x to x+dx-1 do
        begin
          XorPixel13h(i,y,151);
          if dy>1 then
            XorPixel13h(i,y+dy-1,151);
        end;
      for i:=y+1 to y+dy-2 do
        begin
          XorPixel13h(x,i,151);
          XorPixel13h(x+dx-1,i,151);
        end;
    end;


VAR
  F: File;           { PCX file }
  Hdr: PCXHeaderPtr; { PCX header structure & file }
  Pal: PalettePtr;   { PCX palette }
  Shade, Size: Word; { RGB shade, file size }
  var i,j,x,y,dx,dy:integer;
      s:string;
      fout:file of byte;
      key:char;
      b1,b2,c:byte;
      grabbed:boolean;


BEGIN
  Assign(F, '00000005.pcx');         { open PCX file }
  Reset(F,1);
  Size := FileSize(F);
  GetMem(Hdr, Size);                 { load PCX into memory }
  Blockread(F, Hdr^, Size);
  Close(F);
  InitGraph13h;
  Pal := Ptr( Seg(Hdr^), Ofs(Hdr^) + Size - 768);    { get palette location }
  Port[968] := 0;                                    { set palette }
  FOR Shade := 0 TO 767 DO
    Port[969] := Pal^[Shade] SHR 2;
  DisplayPCX(0, 0, Hdr);                             { decode PCX to screen }



  dx:=16; dy:=16; x:=160; y:=100; grabbed:=false;
  DrawFrame(x,y,dx,dy);
  repeat
    key:=ReadKey;
    DrawFrame(x,y,dx,dy);
    case key of
      'q':if y>0 then Dec(y);
      'a':if y+dy-1<199 then Inc(y);
      'o':if x>0 then Dec(x);
      'p':if x+dx-1<319 then Inc(x);
      'Q':if dy>1 then
            begin
              Dec(dy);
              Sound(1760); Delay(5); NoSound;
            end;
      'A':if y+dy-1<199 then
            begin
              Inc(dy);
              Sound(1760); Delay(5); NoSound;
            end;
      'O':if dx>1 then
            begin
              Dec(dx);
              Sound(1760); Delay(5); NoSound;
            end;
      'P':if x+dx-1<319 then
            begin
              Inc(dx);
              Sound(1760); Delay(5); NoSound;
            end;
      #13:begin
            if not grabbed then
              begin
                Assign(fout,'sprites.sps'); Rewrite(fout); grabbed:=true;
              end;
            b1:=dx mod 256; b2:=dx div 256; Write(fout,b1,b2);
            b1:=dy mod 256; b2:=dy div 256; Write(fout,b1,b2);
            for i:=y to y+dy-1 do
              for j:=x to x+dx-1 do
                begin c:=GetPixel13h(j,i); Write(fout,c); end;
            Sound(880); Delay(20); NoSound;
          end;

    end;
    DrawFrame(x,y,dx,dy);
  until key=#27;
  if grabbed then Close(fout);
  CloseGraph13h;
end.


